/*
Copyright (C) 2013 Bengt Martensson.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program. If not, see http://www.gnu.org/licenses/.
 */

package org.harctoolbox.jirc;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.util.LinkedHashMap;
import java.util.Map.Entry;
import org.antlr.runtime.ANTLRFileStream;
import org.antlr.runtime.ANTLRReaderStream;
import org.antlr.runtime.ANTLRStringStream;
import org.antlr.runtime.CommonTokenStream;
import org.antlr.runtime.RecognitionException;
import org.harctoolbox.IrpMaster.IrpUtils;

public class ConfigFile {
    private LinkedHashMap<String, IrRemote> remotes = new LinkedHashMap<String, IrRemote>();

    private ConfigFile(File configFileName, int debug) throws IOException {
        this(new ANTLRFileStream(configFileName.getCanonicalPath()), debug);
    }

    private ConfigFile(InputStream inputStream, int debug) throws IOException {
        this(new ANTLRReaderStream(new InputStreamReader(inputStream, IrpUtils.dumbCharset)), debug);
    }

    private ConfigFile(Reader reader, int debug) throws IOException {
        this(new ANTLRReaderStream(reader), debug);
    }

    private ConfigFile(ANTLRStringStream inputStream, int debug) throws IOException {
       ConfigFileLexer lex = new ConfigFileLexer(inputStream);
        CommonTokenStream tokens = new CommonTokenStream(lex);
        ConfigFileParser parser = new ConfigFileParser(tokens);
        ConfigFileParser.remotes_return r;
        try {
            r = parser.remotes();
            remotes = r.remotes;//parser.remotes();

            IrRemote last = null;
            for (IrRemote rem : remotes.values()) {
                rem.next = null;
                if (last != null)
                    last.next = rem;
                last = rem;
            }
        } catch (RecognitionException ex) {
            System.err.println("Parse error occured, continuing");
            //throw new ParseException(ex);
        }
        //AST = (CommonTree) r.getTree();
    }

    public static LinkedHashMap<String, IrRemote> readConfig(File filename, int debug) throws IOException {
        if (filename.isFile()) {
            ConfigFile config = new ConfigFile(filename, debug);
            return config.remotes;
        } else if (filename.isDirectory()) {
            File[] files = filename.listFiles();
            LinkedHashMap<String, IrRemote>dictionary = new LinkedHashMap<String, IrRemote>();
            for (File file : files) {
                // The program handles nonsensical files fine, however rejecting some
                // obviously irrelevant files saves time and log entries.
                if (file.getName().endsWith(".jpg") || file.getName().endsWith(".gif")) {
                    System.err.println("Rejecting file " + file.getCanonicalPath());
                    continue;
                }
                LinkedHashMap<String, IrRemote> map = readConfig(file, debug);

                for (Entry<String, IrRemote> kvp : map.entrySet()) {
                    if (map.containsKey(kvp.getKey()))
                        System.err.println("Warning: remote name " + kvp.getKey() + " more than once present, taking the last.");
                }
                dictionary.putAll(map);
            }
            return dictionary;
        } else if (!filename.canRead())
            throw new FileNotFoundException(filename.getCanonicalPath());
        else
            return null;
    }

    public static LinkedHashMap<String, IrRemote> readConfig(Reader reader, int debug) throws IOException {
        ConfigFile config = new ConfigFile(reader, debug);
        return config.remotes;
    }

    public static LinkedHashMap<String, IrRemote> readConfig(InputStream inputStream, int debug) throws IOException {
        ConfigFile config = new ConfigFile(inputStream, debug);
        return config.remotes;
    }

    public static LinkedHashMap<String, IrRemote> readConfig(String input, int debug) throws IOException {
        ConfigFile config = new ConfigFile(new ANTLRStringStream(input), debug);
        return config.remotes;
    }
}
